$ifndef _74HC595_
$define _74HC595_
'
' Macros to interface with a 74HC595 serial to parallel shift register
' For use with the Amicus18 BASIC compiler
'
    Include "SPI.inc"                           ' Load the SPI peripheral macros into the program
'
' Choose a pin for the 74HC595 Latch line
'
$ifndef SR74_LAT
    $define SR74_LAT PORTC.0                    ' Connects 74HC595 ST_CP (latch) pin.
$endif

    Dim b74595ShadowPort As Byte System         ' Used to hold a copy of the outputs

    GoTo SR_74HC595_Init                        ' Jump over the subroutines
'****************************************************************************
'* Name    : SR_WriteByte                                                   *
'* Puspose : Write a byte to the 74HC595 shift register                     *
'*                                                                          *
'* Input   : WREG holds the byte to write                                *
'* Output  : None                                                           *
'* Notes   : Creates a pre-processor macro for convenience                  *
'****************************************************************************
$define SR_WriteByte(pValue) WREG = pValue : _SR_WriteByte
_SR_WriteByte Macro
    GoSub __SR_WriteByte
Endm
#ifdef _SR_WriteByte#Req
__SR_WriteByte:
    b74595ShadowPort = WREG
    WriteSPI(b74595ShadowPort)
    Set SR74_LAT
    DelayCS 1
    Clear SR74_LAT
    Return
#endif
'****************************************************************************
'* Name    : SR_WriteBit                                                    *
'* Puspose : Write a bit to the 74HC595 shift register                      *
'*                                                                          *
'* Input   : PRODL holds the bit to write (0 to 7)                          *
'*           WREG holds the state (0 = Off, 1 = On)                         *
'* Output  : None                                                           *
'* Notes   : Uses the shadow port to read the previous state                *
'*           Uses SFR PRODL for parameter passing                           *
'*           Creates a pre-processor macro for convenience                  *
'****************************************************************************
$define SR_WriteBit(pPin,pHiLo) PRODL = pPin : WREG = pHiLo : _SR_WriteBit
_SR_WriteBit Macro
    GoSub __SR_WriteBit
Endm

#ifdef _SR_WriteBit#Req
__SR_WriteBit:
    PRODH = WREG
    LoadBit b74595ShadowPort, PRODL, PRODH
    SR_WriteByte(b74595ShadowPort)
    Return
#endif
'-------------------------------------------------------------------------------------------
SR_74HC595_Init:
    Low SR74_LAT                                 ' Lock the ST_CP (Latch) line
'
' Open the SPI peripheral:
'                           Fosc / 16 (4MHz operation)
'                           SPI mode 0:0
'                           Sample at the end of the clock
    OpenSPI(SPI_FOSC_16, MODE_00, SMPEND)
    b74595ShadowPort = 0
$endif
